/*********************************************************
 *********************************************************/

import java.awt.*;
import java.applet.*;
import java.lang.*;
import java.io.*;
import java.net.*;
import java.util.*;

import webwisdom.tango.*;
import netscape.javascript.JSObject;

/**
 * An interactive test of the Graphics.drawArc and Graphics.fillArc
 * routines. Can be run either as a standalone application by
 * typing "java ArcTest" or as an applet in the AppletViewer.
 */

public class ArcTest extends Applet implements TControlListener, TDataListener {

	TAgent agent = null;
	private int channel = 100;
	private boolean javaOnly = true;
	boolean inTango = true;
	boolean	isMaster = true;
	boolean fromScript = false;

	ArcControls controls;

	int	saved_start;
	int	saved_end;
	int	saved_filled;

	boolean debugApplet = false;

	public void printLine(String printData) {
		if ( debugApplet ) {
			System.out.println(printData);
		}
	}

	public void init() {

		String arg = null;
		try {
			arg = getParameter("interface");
			inTango = true;
		}
		catch(Exception e) {
			inTango = false;
		}

		if (arg == null) {
			inTango = false;
		}
		printLine("<ArcTest> arg : " + arg);
		printLine("<ArcTest> inTango : " + inTango);

		if (inTango) {
			setTango(arg);
		}

		initThis();
		startThis();

	}

	public void initThis() {

		setLayout(new BorderLayout());
		ArcCanvas c = new ArcCanvas();
		add("Center", c);
		add("South", controls = new ArcControls(c, this));

		controls.enable();

		printLine("<ArcTest> initThis()");

		if (inTango) {
			printLine("<ArcTest> Before isMaster");
			isMaster = agent.isMaster();
			printLine("<ArcTest> Before initApplet()");
			initApplet();
		}
	}

	public void initApplet() {
		if ( !isMaster ) {
			String send_data = "Initial";
			sendMessage(send_data);
		}

	}

	public void commJava() {

		try {
			agent = new TAgent(new TLAgentApplet(this));
		}
		catch(TangoException e) {
			e.printStackTrace();
			System.out.println("<ArcTest> agent is not proprely assigned");
		}

	}

	public void commJavaScript() {

		JSObject jsWin = JSObject.getWindow(this);
		Object[] args = new Object[1];
		args[0] = this;
		jsWin.call("appletReadyJS", args);
	}

	public void setTango(String arg) {
                if ( arg.equals("javaApplet") ) {
			fromScript = false;
			commJava();
		}
		else if ( arg.equals("javaScript") ) {
			fromScript = true;
			commJavaScript();
		}

		while ( agent == null ) {
			printLine("<ArcTest> waiting for the agent to be assigned");
		}
		printLine("<ArcTest> agent assigned");

		//dataListener = agent.addTDataListener(channel, this);
		//controlListener = agent.addTControlListener(this);
		agent.addTDataListener(channel, this);
		agent.addTControlListener(this);
		isMaster = agent.isMaster();
	}

	public void setTango(String args[]) {

		try {
			agent = new TAgent(new TLAgentApplication(args));
		}
		catch(TangoException e) {
			e.printStackTrace();
			System.out.println("<ArcTest> agent is not proprely assigned");
		}

		printLine("<ArcTest> Application2 agent assigned");

		agent.addTDataListener(channel, this);
		printLine("<ArcTest> Data added");

		agent.addTControlListener(this);
		printLine("<ArcTest> Control added");

//		isMaster = agent.isMaster();
		printLine("<ArcTest> Data Control added");
	}

	public void initTangoJS(TAgent a, Number ch, Number ma) {
		printLine("<ArcTest> inside initTangoJS()");
		channel = ((Number) ch).intValue();

		if ( ((Number) ma).intValue() ==  0 ) {
			isMaster = true;
		}
		else {
			isMaster = false;
		}

		agent = a;
	}

	public void sendMessage(String send_data) {

		if (!inTango) {
			return;
		}

		printLine("<ArcTest> send : " + send_data);
		agent.send(channel, stringToBytes(send_data));
	}

	public void startThis() {
		printLine("<ArcTest> startThis()");
	}

	public void stop() {
		controls.disable();
	}

	public void destroy() {
		printLine("<ArcTest> inside destroy()");

		if ( fromScript ) {
			agent.removeTControlListener(this);
			agent.removeTDataListener(this);
			printLine("<ArcTest> data and control removed");
			return;
		}

		if (agent != null) {
			agent.exit();
		}
		super.destroy();
	}

	public void closeThis() {
		printLine("<ArcTest> inside closeThis()");
		//agent.removeTControlListener(this);
		//agent.removeTDataListener(this);
		printLine("<ArcTest> DONE closeThis()");
	}

	public boolean handleEvent(Event e) {
		if (e.id == Event.WINDOW_DESTROY) {
			System.exit(0);
		}
		return false;
	}

	public void receive(byte b[]) {
		StringTokenizer recv_data = null ;
		boolean filled;
		int filled_int;
		int start;
		int end;

		String r = bytesToString(b);

		printLine("<ArcTest> receive : " + r);

		// master sends the inital value of filled, start, end
		if (r.startsWith("Initial")) {
			if (isMaster) {
				// make one string data to be sent,
				// convert to byte, and send it
				String send_data = saved_filled + 
					" " + saved_start + " " + saved_end;
				sendMessage(send_data);
			}
			return;
		}

		// parse the received String
		recv_data = new StringTokenizer(r);

		filled_int = (Integer.valueOf(recv_data.nextToken())).intValue();
		start      = (Integer.valueOf(recv_data.nextToken())).intValue();
		end        = (Integer.valueOf(recv_data.nextToken())).intValue();

		if ( filled_int == 1 ) {
			filled = true;
		}
		else {
			filled = false;
		}

		controls.draw_canvas(filled, start, end);
		controls.s.setText(String.valueOf(start));
		controls.e.setText(String.valueOf(end));

	}

	private byte[] stringToBytes(String s) {
		int l = s.length();
		byte[] t = new byte[l];
		s.getBytes(0, l, t, 0);
		return t;
	}

	private String bytesToString(byte t[]) {
		return new String(t, 0);
	}

	public void masterChanged(boolean isMaster, String masterName) {
	}

	public void participantJoined(String participantName) {

	}

	public void participantLeft(String participantName) {

	}

	public void audioRequested(boolean isREquested) {
	}

	public static void main(String args[]) {
		Frame f = new Frame("ArcTest");
		ArcTest arcTest = new ArcTest();

		try {
			arcTest.printLine("<ArcTest> arg[0] : " + args[0]);
			arcTest.inTango = true;
		}
		catch (Exception e) {
			arcTest.inTango = false;
		}

		arcTest.printLine("inTango : " + arcTest.inTango);

		if (arcTest.inTango) {
			arcTest.setTango(args);
		}

		arcTest.initThis();
		arcTest.startThis();

		f.add("Center", arcTest);
		f.setSize(400, 440);
		f.show();
	}
}

class ArcCanvas extends Canvas {
	int		startAngle = 0;
	int		endAngle = 45;
	boolean	filled = false;
	Font	font;

	public void paint(Graphics g) {
		Rectangle r = bounds();
		int hlines = r.height / 10;
		int vlines = r.width / 10;

		g.setColor(Color.pink);
		for (int i = 1; i <= hlines; i++) {
			g.drawLine(0, i * 10, r.width, i * 10);
		}
		for (int i = 1; i <= vlines; i++) {
			g.drawLine(i * 10, 0, i * 10, r.height);
		}

		g.setColor(Color.red);
		if (filled) {
			g.fillArc(0, 0, r.width - 1, r.height - 1, startAngle, endAngle);
		}
		else {
			g.drawArc(0, 0, r.width - 1, r.height - 1, startAngle, endAngle);
		}

		g.setColor(Color.black);
		g.setFont(font);
		g.drawLine(0, r.height / 2, r.width, r.height / 2);
		g.drawLine(r.width / 2, 0, r.width / 2, r.height);
		g.drawLine(0, 0, r.width, r.height);
		g.drawLine(r.width, 0, 0, r.height);
		int sx = 10;
		int sy = r.height - 28;
		g.drawString("S = " + startAngle, sx, sy); 
		g.drawString("E = " + endAngle, sx, sy + 14); 
	}

	public void redraw(boolean filled, int start, int end) {
		this.filled = filled;
		this.startAngle = start;
		this.endAngle = end;
		repaint();
	}
}

class ArcControls extends Panel {
	TextField s;
	TextField e;
	ArcCanvas canvas;
	ArcTest test;

	public ArcControls(ArcCanvas canvas, ArcTest test) {
		this.canvas = canvas;
		this.test = test;
		add(s = new TextField("0", 4));
		add(e = new TextField("45", 4));
		test.saved_start	= 0;
		test.saved_end		= 45;
		test.saved_filled	= 0;
		add(new Button("Fill"));
		add(new Button("Draw"));
	}

	public boolean action(Event ev, Object arg) {

		String send_data = null;
		int button_action;
		int int_start;
		int int_end;
		boolean filled;

		if (ev.target instanceof Button) {
			String label = (String)arg;

			filled = label.equals("Fill");
			if (filled) {
				button_action = 1;
			}
			else {
				button_action = 0;
			}

			int_start = Integer.parseInt(s.getText().trim());
			int_end   = Integer.parseInt(e.getText().trim());

			test.saved_start	= int_start;
			test.saved_end		= int_end;
			test.saved_filled	= button_action;

			// make one string data to be sent,
			// convert to byte, and send it
			send_data = button_action + " " + int_start + " " + int_end;

			test.sendMessage(send_data);

			canvas.redraw(filled, int_start, int_end);

			return true;
		}

		return false;
	}

	public void draw_canvas(boolean filled, int start, int end) {
		canvas.redraw(filled, start, end);
	}
}
	
